


//////////////////////////////////////////////////////////////////
//                                                              //
//           PLINK (c) 2005-2006 Shaun Purcell                  //
//                                                              //
// This file is distributed under the GNU General Public        //
// License, Version 2.  Please see the file COPYING for more    //
// details                                                      //
//                                                              //
//////////////////////////////////////////////////////////////////


#include <iostream>
#include <iomanip>
#include <fstream>
#include <sstream>
#include <cmath>
#include "plink.h"
#include "options.h"
#include "perm.h"


void Plink::findSegments(int i1, int i2, vector_t & p, ofstream & SEG)
{

  Individual * p1 = sample[i1];
  Individual * p2 = sample[i2];
  
  bool inseg = false;
  
  Segment s;
  s.p1 = p1;
  s.p2 = p2;
  int npos = p.size();
		
  // Marker if minimal segment output file is selected
  bool already_seen_pair = false;
  
  for (int l=0; l<npos; l++)
    {
      
      ///////////////////////////
      // Start of a new segment?
      
      if ( (!inseg) && p[l] >= par::segment_threshold_start)
	{
	  inseg = true;

	  if (m1[l] != -1)
	    s.start = m1[l];
	  else
	    s.start = par::run_start;
	}
      
      
        ///////////////////////////
	// End of existing segment?
		    		    
	if ( inseg && 
	     ( p[l] < par::segment_threshold_finish || l == npos-1 ) )
	  {
	    inseg = false;

	    if (m2[l-1] != -1)
	      s.finish = m2[l-1];
	    else
	      s.finish = par::run_end;

// 	    cout << "\nSEG dets = " << locus[s.finish]->bp - locus[s.start]->bp << "\n";

// 	    cout << "flanking SNPs\n";

// 	    if ( s.start > 0 && s.start < nl_all -1 ) 
// 	      cout << locus[s.start-1]->name << " [ " 
// 		   << locus[s.start]->name << " ] " 
// 		   << locus[s.start+1]->name << "\n"
// 		   << p[l-1] << " "
// 		   << p[l] << " "
// 		   << p[l+1] << "\n";
	    
// 	    if ( s.finish > 0 && s.finish < nl_all -1 )
// 	      cout << locus[s.finish-1]->name << " [ " 
// 		   << locus[s.finish]->name << " ] " 
// 		   << locus[s.finish+1]->name << "\n"
// 		   << p[l-1] << " "
// 		   << p[l] << " "
// 		   << p[l+1] << "\n";
// 	    cout << "\n";

	    // Do we like this segment?
	    if ( locus[s.finish]->bp - locus[s.start]->bp 
		 >= par::segment_length)
	      {
		
		// Add segment to list
		segment.push_back(s);
		
		// Display?
		if (par::segment_output)
		  {
		    
		    if ( par::segment_minimal )
		      {
			
			if ( ! already_seen_pair )
			  {
			    already_seen_pair = true;
			    SEG << i1 << " " << i2 << " ";
			  }
			
			SEG << s.start << " "
			    << s.finish << " ";
		      }
		    else
		      {
			
			// Long format
			
			SEG << setw(10) << s.p1->fid << " "
			    << setw(10) << s.p1->iid << " "
			    << setw(10) << s.p2->fid << " "
			    << setw(10) << s.p2->iid << " ";
			
			if (par::bt)
			  {
			    if ( (!p1->aff) && (!p2->aff) ) 
			      SEG << setw(4) << "-1" << " ";
			    else if ( p1->aff && p2->aff )
			      SEG << setw(4) << "1" << " ";
			    else if ((!p1->aff) && p2->aff)
			      SEG << setw(4) << "0" << " ";
			    else if (p1->aff && !p2->aff)
			      SEG << setw(4) << "0" << " ";
			    else
			      SEG << setw(4) << "NA" << " ";
			  }
			else
			  SEG << setw(4) << "NA" << " ";
			
			Locus * start = locus[s.start];
			Locus * finish = locus[s.finish];
			
			SEG << setw(4) << par::run_chr << " "
			    << setw(10) << start->bp << " "
			    << setw(10) << finish->bp << " "
			    << setw(par::pp_maxsnp) << start->name << " "
			    << setw(par::pp_maxsnp) << finish->name << " "
			    << setw(6) << s.finish-s.start+1 << " "
			    << setw(10) << (double)(finish->bp - start->bp)/(double)1000 
			    << "\n";
		      }
		  }
		
		
	      }
	    
	  }
    }

  // If we have found segments for this pair, then 
   // record a end-of-pair code if in minimal output mode

  if ( par::segment_minimal && already_seen_pair )
    SEG << "-1 -1\n";

}



void Plink::segmentPermutationTest(Perm & perm, bool ibd, string f,
				   vector<int> & coverage_conc_aff,
				   vector<int> & coverage_disc,
				   vector<int> & coverage_conc_unaff )
{
  

  // IBD or IBS segment permutation test for case/control data
  // Calculate total number of concordant/discordant pairs
  
  int tot_conc_aff=0;
  int tot_not=0;
  
  for (int i1=0; i1<n-1; i1++)
    for (int i2=i1+1; i2<n; i2++)
      if ( sample[i1]->aff && sample[i2]->aff ) tot_conc_aff++;
      else tot_not++;
  
  printLOG(int2str(tot_conc_aff)+" concordant affected pairs out of "
	   +int2str(tot_not+tot_conc_aff)+" in total\n");
  

  int nt = nl_all;  // IBS mode
  if (ibd) { nt = nl; } 
  
  perm.setTests(nt);
  perm.setPermClusters(*this);
  perm.originalOrder();

  vector<double> original(nt);
  
  // Get genome-wide means

  double SCg = 0;
  double SDg = 0;
  for (int l=0; l<nt; l++)
    {
      SCg += coverage_conc_aff[l];
      SDg += coverage_disc[l] + coverage_conc_unaff[l];
    }

  SCg /= (double)nt;
  SDg /= (double)nt;
 
  for (int l=0; l<nt; l++)
    {
      
      double SC = coverage_conc_aff[l];
      double SD = coverage_disc[l] + coverage_conc_unaff[l];

      // Correct for genome-wide average
      SC = SC-SCg < 0 ? 0 : SC - SCg;
      SD = SD-SDg < 0 ? 0 : SD - SDg;

      double statistic = SC / (double)tot_conc_aff  -  SD / (double)tot_not ;
      original[l] = statistic < 0 ? 0 : statistic;
	
    }
    

    //////////////////////
    // Begin permutations
    
    // Edit 'fringe' status if in IBD mode
    
     if (ibd)
       {
 	vector<Segment>::iterator s = segment.begin();
 	while ( s != segment.end() )
 	  {
 	    
	    if ( s->start == -1 ) s->start = par::run_start;
 	    if ( s->finish == -1 ) s->finish = par::run_end;
 	    
	    // Edit to make positions relative to start of this chromosome
 	    s->start -= par::run_start;
 	    s->finish -= par::run_start;
	    
	    s++;
 	  }
       }

    bool finished = false;
    while(!finished)
      {
	
	// Store permuted results
	
	vector<double> pr(nt);
	
	// Permute
	
	perm.permuteInCluster();
	
	// Retest
	
	vector<int> coverage_conc_aff(nt,0);
	vector<int> coverage_conc_unaff(nt,0);
	vector<int> coverage_disc(nt,0);
	
	vector<Segment>::iterator s = segment.begin();
	
	while ( s != segment.end() )
	  {    
	    
	    if (s->p1->pperson->aff == s->p2->pperson->aff)
	      {
		if ( s->p1->pperson->aff) 
		  for (int l = s->start ; l <= s->finish; l++) coverage_conc_aff[l]++;
		else
		  for (int l = s->start ; l <= s->finish; l++) coverage_conc_unaff[l]++;
	      }
	    else
	      for (int l = s->start ; l <= s->finish; l++) coverage_disc[l]++;
	    
	    s++;
	  }
	
	double SCg = 0;
	double SDg = 0;
	for (int l=0; l<nt; l++)
	  {
	    SCg += coverage_conc_aff[l];
	    SDg += coverage_disc[l] + coverage_conc_unaff[l];
	  }
	
	SCg /= (double)nt;
	SDg /= (double)nt;

	for (int l=0; l<nt; l++)
	  {
	    
	    double SC = coverage_conc_aff[l];
	    double SD = coverage_disc[l] + coverage_conc_unaff[l];

	    // Correct for genome-wide average
	    SC = (SC-SCg < 0) ? 0 : SC - SCg;
	    SD = (SD-SDg < 0) ? 0 : SD - SDg;
    
	    double statistic = SC / (double)tot_conc_aff - (double)SD / tot_not ;
	    pr[l] = statistic < 0 ? 0 : statistic;
	    	    	    	    
	  }
	
	
	
	////////////////////////////////
	// Standard permutation counting
	
	finished = perm.update(pr,original);
	
      }
    
    if (!par::silent)
      cout << "\n\n";
  
  
    ////////////////////////////
    // Display permuted p-values
    
    f += ".mperm";
    printLOG("Writing permuted results for segment test to [ "+f+" ]\n");
    
    ofstream SIBS;
    SIBS.open( f.c_str() , ios::out );
    
    for (int l=0; l<nt; l++)
      {	
	
	if (!ibd)
	  SIBS << setw(4)  << locus[l]->chr << " " 
	       << setw(par::pp_maxsnp) << locus[l]->name << " ";
	  
	  SIBS << setw(12) << original[l]  << " " 
	       << setw(12) << perm.pvalue(l) << " "
	       << setw(12) << perm.max_pvalue(l) << "\n";      
	
    }

  SIBS.close();

}



void Plink::testGenomeIBDByCovariate(Perm & perm)
{
  
  // Calculate case/case, control/control and case/control
  // Comparisons:

  //   1) Case/case versus all others
  //   2) Case/control versus all others
  //   3) Control/control versus all others

  //   4) Case/case versus case/control
  //   5) Control/Control versus case/control
  //   6) Case/case versus control/control

  // Test based on yes/no dichotomy for % pairs above certain PIHAT threshold 
  
  int tot_conc_aff=0;
  int tot_conc_unaff=0;
  int tot_disc=0;
  
  for (int i1=0; i1<n-1; i1++)
    for (int i2=i1+1; i2<n; i2++)
      {
	if ( sample[i1]->aff )
	  {
	    if ( sample[i2]->aff ) 
	      tot_conc_aff++;
	    else tot_disc++;
	  }
	else
	  {
	    if ( sample[i2]->aff ) 
	      tot_disc++;
	    else tot_conc_unaff++;
	  }
      }

  printLOG(int2str(tot_conc_aff)+" concordant affected pairs\n");
  printLOG(int2str(tot_conc_unaff)+" concordant unaffected pairs\n");
  printLOG(int2str(tot_disc)+" discordant pairs\n");
  printLOG(int2str(tot_conc_aff+tot_conc_unaff+tot_disc)+" total pairs\n");
  
  int nt = 6;  // six tests (above)

  perm.setTests(nt);
  perm.setPermClusters(*this);
  perm.originalOrder();


  ///////////////////////////
  // Original test statistics

  vector<double> original(nt);

  
  // Iterate over all pairs
  
  double prop_11 = 0;
  double prop_01 = 0;
  double prop_00 = 0;

  int c=0;
  for (int i1=0; i1<n-1; i1++)
    for (int i2=i1+1; i2<n; i2++)
      {
	Individual * p1 = sample[i1];
	Individual * p2 = sample[i2];
	
	int2 pair;
	pair.p1 = i1;
	pair.p2 = i2;

	// 	cout << c << " of " << saved_IBDg.size() << " check : \n";
	// 	cout << saved_IBDg[c].z1/2+saved_IBDg[c].z2 << "\n";
	// 	c++;
	//	cout <<"\n";


	if ( related.find(pair) != related.end() )
	  {
	    if ( sample[i1]->aff )
	      {
		if ( sample[i2]->aff ) 
		  prop_11++;
		else prop_01++;
	      }
	    else
	      {
		if ( sample[i2]->aff ) 
		  prop_01++;
		else prop_00++;
	      }
	  }
	
      }
  
  //   1) Case/case versus all others
  //   2) Case/control versus all others
  //   3) Control/control versus all others

  //   4) Case/case versus case/control
  //   5) Control/Control versus case/control
  //   6) Case/case versus control/control

  
  original[0] = (double)prop_11/(double)tot_conc_aff -   double(prop_01+prop_00)/(double)(tot_disc+tot_conc_unaff) ;
  original[1] = (double)prop_01/(double)tot_disc -       double(prop_11+prop_00)/(double)(tot_conc_aff+tot_conc_unaff) ;
  original[2] = (double)prop_00/(double)tot_conc_unaff - double(prop_01+prop_11)/(double)(tot_disc+tot_conc_aff) ;
  
  original[3] = (double)prop_11/(double)tot_conc_aff -   double(prop_01)/(double)(tot_disc) ;
  original[4] = (double)prop_00/(double)tot_conc_unaff - double(prop_01)/(double)(tot_disc) ;
  original[5] = (double)prop_11/(double)tot_conc_aff -   double(prop_00)/(double)(tot_conc_unaff) ;


  ////////////////////
  // Begin permutations
  
  bool finished = false;
  while(!finished)
    {
      
      // Store permuted results
      
      vector<double> pr(nt,0);
      
      // Permute
      
      perm.permuteInCluster();
      
      // Retest
      
      double prop_11 = 0;
      double prop_01 = 0;
      double prop_00 = 0;
      
      for (int i1=0; i1<n-1; i1++)
	for (int i2=i1+1; i2<n; i2++)
	  {
	    Individual * p1 = sample[i1];
	    Individual * p2 = sample[i2];
	    
	    int2 pair;
	    pair.p1 = i1;
	    pair.p2 = i2;
	    
	    if ( related.find(pair) != related.end() )
	      {
		if ( sample[i1]->pperson->aff )
		  {
		    if ( sample[i2]->pperson->aff ) 
		      prop_11++;
		    else prop_01++;
		  }
		else
		  {
		    if ( sample[i2]->pperson->aff ) 
		      prop_01++;
		    else prop_00++;
		  }
	      }
	    
	  }
      
      //   1) Case/case versus all others
      //   2) Case/control versus all others
      //   3) Control/control versus all others
      
      //   4) Case/case versus case/control
      //   5) Control/Control versus case/control
      //   6) Case/case versus control/control

      pr[0] = (double)prop_11/(double)tot_conc_aff -   double(prop_01+prop_00)/(double)(tot_disc+tot_conc_unaff) ;
      pr[1] = (double)prop_01/(double)tot_disc -       double(prop_11+prop_00)/(double)(tot_conc_aff+tot_conc_unaff) ;
      pr[2] = (double)prop_00/(double)tot_conc_unaff - double(prop_01+prop_11)/(double)(tot_disc+tot_conc_aff) ;
      
      pr[3] = (double)prop_11/(double)tot_conc_aff -   double(prop_01)/(double)(tot_disc) ;
      pr[4] = (double)prop_00/(double)tot_conc_unaff - double(prop_01)/(double)(tot_disc) ;
      pr[5] = (double)prop_11/(double)tot_conc_aff -   double(prop_00)/(double)(tot_conc_unaff) ;
      	
      
      ////////////////////////////////
      // Standard permutation counting
      
      finished = perm.update(pr,original);
	
      }
    
    if (!par::silent)
      cout << "\n\n";
  
  
    ////////////////////////////
    // Display permuted p-values
    
    string f = par::output_file_name + ".genome.mperm";
    printLOG("Writing permuted results for genome-wide IBD test to [ "+f+" ]\n");
    
    ofstream SIBS;
    SIBS.open( f.c_str() , ios::out );
    
    for (int l=0; l<nt; l++)
      {	
	
	SIBS << setw(4) << l << " "  
	     << setw(12) << original[l]  << " " 
	     << setw(12) << perm.pvalue(l) << " "
	     << setw(12) << perm.max_pvalue(l) << "\n";      
	
    }

  SIBS.close();

}

